@@ -23,7 +23,7 @@ from conda.cli.common import specs_from_url, confirm_yn, check_non_admin, ensure
 from conda.core.subdir_data import SubdirData
 from conda.core.link import UnlinkLinkTransaction, PrefixSetup
 from conda.cli.install import check_prefix, clone, print_activate
-from conda.base.constants import ChannelPriority, ROOT_ENV_NAME, UpdateModifier
+from conda.base.constants import ChannelPriority, ROOT_ENV_NAME, UpdateModifier, DepsModifier
 from conda.core.solve import diff_for_unlink_link_precs
 from conda.core.envs_manager import unregister_env
 from conda.core.package_cache_data import PackageCacheData
@@ -49,15 +49,15 @@ import json
 import tempfile
 import logging
 
+import mamba
 import mamba.mamba_api as api
 
-from mamba.post_solve_handling import post_solve_handling
-from mamba.utils import get_index, to_package_record_from_subjson
+from mamba.utils import get_index, to_package_record_from_subjson, init_api_context
 
 log = getLogger(__name__)
 stderrlog = getLogger('mamba.stderr')
 
-banner = """
+banner = f"""
                   __    __    __    __
                  /  \\  /  \\  /  \\  /  \\
                 /    \\/    \\/    \\/    \\
@@ -73,7 +73,7 @@ banner = """
         ██║ ╚═╝ ██║██║  ██║██║ ╚═╝ ██║██████╔╝██║  ██║
         ╚═╝     ╚═╝╚═╝  ╚═╝╚═╝     ╚═╝╚═════╝ ╚═╝  ╚═╝
 
-        Supported by @QuantStack
+        mamba ({mamba.__version__}) supported by @QuantStack
 
         GitHub:  https://github.com/QuantStack/mamba
         Twitter: https://twitter.com/QuantStack
@@ -81,21 +81,6 @@ banner = """
 █████████████████████████████████████████████████████████████
 """
 
-def init_api_context():
-    api_ctx = api.Context()
-    api_ctx.set_verbosity(context.verbosity)
-    api_ctx.quiet = context.quiet
-    api_ctx.json = context.json
-    api_ctx.offline = context.offline
-    api_ctx.local_repodata_ttl = context.local_repodata_ttl
-    api_ctx.use_index_cache = context.use_index_cache
-    api_ctx.always_yes = context.always_yes
-    api_ctx.dry_run = context.dry_run
-    if context.ssl_verify == False:
-        api_ctx.ssl_verify = "<false>"
-    elif context.ssl_verify is not True:
-        api_ctx.ssl_verify = context.ssl_verify 
-
 class MambaException(Exception):
     pass
 
@@ -170,7 +155,6 @@ def to_txn(specs_to_add, specs_to_remove, prefix, to_link, to_unlink, index=None
         final_precs.add(rec)
         to_link_records.append(rec)
 
-    final_precs, specs_to_add, specs_to_remove = post_solve_handling(context, prefix_data, final_precs, specs_to_add, specs_to_remove)
     unlink_precs, link_precs = diff_for_unlink_link_precs(prefix,
                                                           final_precs=IndexedSet(PrefixGraph(final_precs).graph),
                                                           specs_to_add=specs_to_add,
@@ -188,7 +172,7 @@ def to_txn(specs_to_add, specs_to_remove, prefix, to_link, to_unlink, index=None
     conda_transaction = UnlinkLinkTransaction(pref_setup)
     return conda_transaction
 
-use_mamba_download = False
+use_mamba_experimental = False
 
 def handle_txn(unlink_link_transaction, prefix, args, newenv, remove_op=False):
     if unlink_link_transaction.nothing_to_do:
@@ -198,18 +182,12 @@ def handle_txn(unlink_link_transaction, prefix, args, newenv, remove_op=False):
         elif not newenv:
             if context.json:
                 cli_common.stdout_json_success(message='All requested packages already installed.')
-            else:
-                print('\n# All requested packages already installed.\n')
             return
 
-    # don't confirm / print if using mamba for download
-    if not context.json and not use_mamba_download:
-        unlink_link_transaction.print_transaction_summary()
-        confirm_yn()
-
-    elif context.dry_run:
+    if context.dry_run:
         actions = unlink_link_transaction._make_legacy_action_groups()[0]
-        cli_common.stdout_json_success(prefix=prefix, actions=actions, dry_run=True)
+        if context.json:
+            cli_common.stdout_json_success(prefix=prefix, actions=actions, dry_run=True)
         raise DryRunExit()
 
     try:
@@ -289,10 +267,8 @@ def remove(args, parser):
             specs = tuple(MatchSpec(track_features=f) for f in set(args.package_names))
         else:
             specs = [s for s in specs_from_args(args.package_names)]
-        if not (context.quiet or context.json):
-            print("Removing specs: {}".format(specs))
-        channel_urls = ()
-        subdirs = ()
+        if not context.quiet:
+            print("Removing specs: {}".format([s.conda_build_form() for s in specs]))
 
         installed_json_f = get_installed_jsonfile(prefix)
 
@@ -304,9 +280,15 @@ def remove(args, parser):
         repos = []
 
         # add installed
-        repo = api.Repo(pool, "installed", installed_json_f.name, "")
-        repo.set_installed()
-        repos.append(repo)
+        if use_mamba_experimental:
+            prefix_data = api.PrefixData(context.target_prefix)
+            prefix_data.load()
+            repo = api.Repo(pool, prefix_data)
+            repos.append(repo)
+        else:
+            repo = api.Repo(pool, "installed", installed_json_f.name, "")
+            repo.set_installed()
+            repos.append(repo)
 
         solver = api.Solver(pool, solver_options)
         solver.add_jobs(mamba_solve_specs, api.SOLVER_ERASE)
@@ -316,10 +298,19 @@ def remove(args, parser):
             exit_code = 1
             return exit_code
 
-        transaction = api.Transaction(solver)
-        to_link, to_unlink = transaction.to_conda()
+        package_cache = api.MultiPackageCache(context.pkgs_dirs)
+        transaction = api.Transaction(solver, package_cache)
+        downloaded = transaction.prompt(PackageCacheData.first_writable().pkgs_dir, repos)
+        if not downloaded:
+            exit(0)
 
-        conda_transaction = to_txn((), specs, prefix, to_link, to_unlink)
+        mmb_specs, to_link, to_unlink = transaction.to_conda()
+        transaction.log_json()
+
+        specs_to_add = [MatchSpec(m) for m in mmb_specs[0]]
+        specs_to_remove = [MatchSpec(m) for m in mmb_specs[1]]
+
+        conda_transaction = to_txn(specs_to_add, specs_to_remove, prefix, to_link, to_unlink)
         handle_txn(conda_transaction, prefix, args, False, True)
 
 def install(args, parser, command='install'):
@@ -329,7 +320,7 @@ def install(args, parser, command='install'):
     context.validate_configuration()
     check_non_admin()
 
-    init_api_context()
+    init_api_context(use_mamba_experimental)
 
     newenv = bool(command == 'create')
     isinstall = bool(command == 'install')
@@ -375,16 +366,11 @@ def install(args, parser, command='install'):
             else:
                 raise EnvironmentLocationNotFound(prefix)
 
-    prepend = not args.override_channels
     prefix = context.target_prefix
 
-    index_args = {
-        'use_cache': args.use_index_cache,
-        'channel_urls': context.channels,
-        'unknown': args.unknown,
-        'prepend': not args.override_channels,
-        'use_local': args.use_local
-    }
+    #############################
+    # Get SPECS                 #
+    #############################
 
     args_packages = [s.strip('"\'') for s in args.packages]
     if newenv and not args.no_default_packages:
@@ -405,6 +391,38 @@ def install(args, parser, command='install'):
             raise CondaValueError("cannot mix specifications with conda package"
                                   " filenames")
 
+    specs = []
+
+    if args.file:
+        file_specs = []
+        for fpath in args.file:
+            try:
+                file_specs += specs_from_url(fpath, json=context.json)
+            except Unicode:
+                raise CondaValueError("Error reading file, file should be a text file containing"
+                                 " packages \nconda create --help for details")
+        if '@EXPLICIT' in file_specs:
+            explicit(file_specs, prefix, verbose=not (context.quiet or context.json), index_args=index_args)
+            return
+        specs.extend([MatchSpec(s) for s in file_specs])
+
+    specs.extend(specs_from_args(args_packages, json=context.json))
+
+    channels = [c for c in context.channels]
+    for spec in specs:
+        # CONDA TODO: correct handling for subdir isn't yet done
+        spec_channel = spec.get_exact_value('channel')
+        if spec_channel and spec_channel not in channels:
+            channels.append(spec_channel)
+
+    index_args = {
+        'use_cache': args.use_index_cache,
+        'channel_urls': channels,
+        'unknown': args.unknown,
+        'prepend': not args.override_channels,
+        'use_local': args.use_local
+    }
+
     index = get_index(channel_urls=index_args['channel_urls'],
                       prepend=index_args['prepend'], platform=None,
                       use_local=index_args['use_local'], use_cache=index_args['use_cache'],
@@ -431,28 +449,19 @@ def install(args, parser, command='install'):
 
         subpriority = 0 if chan.platform == 'noarch' else 1
 
+        if subdir.loaded() == False and chan.platform != 'noarch':
+            # ignore non-loaded subdir if channel is != noarch
+            continue
+
         if context.verbosity != 0:
             print("Cache path: ", subdir.cache_path())
         channel_json.append((chan, subdir.cache_path(), priority, subpriority))
 
+    # for c in channel_json:
+    #     print(c[0], c[2], c[3])
+
     installed_json_f = get_installed_jsonfile(prefix)
 
-    specs = []
-
-    if args.file:
-        for fpath in args.file:
-            try:
-                file_specs = specs_from_url(fpath, json=context.json)
-            except Unicode:
-                raise CondaValueError("Error reading file, file should be a text file containing"
-                                 " packages \nconda create --help for details")
-        if '@EXPLICIT' in file_specs:
-            explicit(file_specs, prefix, verbose=not (context.quiet or context.json), index_args=index_args)
-            return
-        specs.extend([MatchSpec(s) for s in file_specs])
-
-    specs.extend(specs_from_args(args_packages, json=context.json))
-
     if isinstall and args.revision:
         get_revision(args.revision, json=context.json)
     elif isinstall and not (args.file or args_packages):
@@ -486,29 +495,37 @@ def install(args, parser, command='install'):
         print_activate(args.name if args.name else prefix)
         return
 
+    spec_names = [s.name for s in specs]
+
+    if not (context.quiet or context.json):
+        print("\nLooking for: {}\n".format(spec_names))
+
     # If python was not specified, check if it is installed.
     # If yes, add the installed python to the specs to prevent updating it.
-    spec_names = [s.name for s in specs]
+    python_constraint = None
     if 'python' not in spec_names:
         installed_names = [i_rec.name for i_rec in installed_pkg_recs]
         if 'python' in installed_names:
             i = installed_names.index('python')
             version = installed_pkg_recs[i].version
-            specs.append(MatchSpec('python==' + version))
+            python_constraint = MatchSpec('python==' + version).conda_build_form()
 
-    mamba_solve_specs = [s.conda_build_form() for s in specs]
-
-    if not (context.quiet or context.json):
-        print("\nLooking for: {}\n".format(mamba_solve_specs))
+    mamba_solve_specs = [s.__str__() for s in specs]
 
     pool = api.Pool()
 
     repos = []
 
     # add installed
-    repo = api.Repo(pool, "installed", installed_json_f.name, "")
-    repo.set_installed()
-    repos.append(repo)
+    if use_mamba_experimental:
+        prefix_data = api.PrefixData(context.target_prefix)
+        prefix_data.load()
+        repo = api.Repo(pool, prefix_data)
+        repos.append(repo)
+    else:
+        repo = api.Repo(pool, "installed", installed_json_f.name, "")
+        repo.set_installed()
+        repos.append(repo)
 
     for channel, cache_file, priority, subpriority in channel_json:
         repo = api.Repo(pool, str(channel), cache_file, channel.url(with_credentials=True))
@@ -517,23 +534,42 @@ def install(args, parser, command='install'):
 
     solver = api.Solver(pool, solver_options)
     solver.add_jobs(mamba_solve_specs, solver_task)
+
+    if python_constraint:
+        solver.add_constraint(python_constraint)
+
+    solver.set_postsolve_flags(
+        [(api.MAMBA_NO_DEPS, context.deps_modifier == DepsModifier.NO_DEPS), 
+         (api.MAMBA_ONLY_DEPS, context.deps_modifier == DepsModifier.ONLY_DEPS)]
+    )
     success = solver.solve()
     if not success:
         print(solver.problems_to_str())
         exit_code = 1
         return exit_code
 
-    transaction = api.Transaction(solver)
-    to_link, to_unlink = transaction.to_conda()
+    package_cache = api.MultiPackageCache(context.pkgs_dirs)
+    transaction = api.Transaction(solver, package_cache)
+    mmb_specs, to_link, to_unlink = transaction.to_conda()
 
-    if use_mamba_download:
-        downloaded = transaction.prompt(PackageCacheData.first_writable().pkgs_dir, repos)
-        if not downloaded:
-            exit(0)
-        PackageCacheData.first_writable().reload()
+    specs_to_add = [MatchSpec(m) for m in mmb_specs[0]]
+    specs_to_remove = [MatchSpec(m) for m in mmb_specs[1]]
 
-    conda_transaction = to_txn(specs, (), prefix, to_link, to_unlink, index)
-    handle_txn(conda_transaction, prefix, args, newenv)
+    transaction.log_json()
+
+    downloaded = transaction.prompt(PackageCacheData.first_writable().pkgs_dir, repos)
+    if not downloaded:
+        exit(0)
+    PackageCacheData.first_writable().reload()
+
+    if use_mamba_experimental and not os.name == 'nt':
+        if newenv and not isdir(context.target_prefix) and not context.dry_run:
+            mkdir_p(prefix)
+
+        transaction.execute(prefix_data, PackageCacheData.first_writable().pkgs_dir)
+    else:
+        conda_transaction = to_txn(specs_to_add, specs_to_remove, prefix, to_link, to_unlink, index)
+        handle_txn(conda_transaction, prefix, args, newenv)
 
     try:
         installed_json_f.close()
@@ -571,7 +607,6 @@ def update(args, parser):
     install(args, parser, 'update')
 
 def repoquery(args, parser):
-    prepend = not args.override_channels
     prefix = context.target_prefix
 
     init_api_context()
@@ -584,11 +619,6 @@ def repoquery(args, parser):
         'use_local': args.use_local
     }
 
-    index = get_index(channel_urls=index_args['channel_urls'],
-                  prepend=index_args['prepend'], platform=None,
-                  use_local=index_args['use_local'], use_cache=index_args['use_cache'],
-                  unknown=index_args['unknown'], prefix=prefix)
-
     installed_json_f = get_installed_jsonfile(prefix)
 
     pool = api.Pool()
@@ -599,20 +629,35 @@ def repoquery(args, parser):
     repo.set_installed()
     repos.append(repo)
 
-    if not args.installed:
+    only_installed = True
+    if args.subcmd == "search" and args.installed == False:
+        only_installed = False
+
+    if not only_installed:
+        index = get_index(channel_urls=index_args['channel_urls'],
+                      prepend=index_args['prepend'], platform=None,
+                      use_local=index_args['use_local'], use_cache=index_args['use_cache'],
+                      unknown=index_args['unknown'], prefix=prefix)
+
         for subdir, channel in index:
+            if subdir.loaded() == False and channel.platform != 'noarch':
+                # ignore non-loaded subdir if channel is != noarch
+                continue
+
             repo = api.Repo(pool, str(channel), subdir.cache_path(), channel.url(with_credentials=True))
             repo.set_priority(0, 0)
             repos.append(repo)
 
-
-    print("\nExecuting the query %s\n" % args.query)
+    if not context.json:
+        print("\nExecuting the query %s\n" % args.package_query)
 
     query = api.Query(pool)
-    if args.whatrequires:
-        print(query.whatrequires(args.query))
-    else:
-        print(query.find(args.query))
+    if args.subcmd == "whoneeds":
+        query.whoneeds(args.package_query, args.tree)
+    if args.subcmd == "depends":
+        query.depends(args.package_query)
+    if args.subcmd == "search":
+        query.find(args.package_query)
 
 
 def do_call(args, parser):
@@ -646,58 +691,73 @@ def configure_parser_repoquery(sub_parsers):
     example = ("""
 Examples:
 
-    conda repoquery xtensor>=0.18
+    mamba repoquery search xtensor>=0.18
+    mamba repoquery depends xtensor
+    mamba repoquery whoneeds xtl
 
     """)
 
+    import argparse
     from argparse import SUPPRESS
 
     p = sub_parsers.add_parser(
         'repoquery',
         description=descr,
         help=help,
-        epilog=example,
+        epilog=example
     )
-
-    p.add_argument(
-        'query',
-        action="store",
-        nargs='?',
-        help="Package query.",
-    )
-
-    p.add_argument(
-        "--whatrequires",
+    subsub_parser = p.add_subparsers(dest='subcmd')
+    package_cmds = argparse.ArgumentParser(add_help=False)
+    package_cmds.add_argument('package_query', help='the target package')
+    package_cmds.add_argument(
+        "-i", "--installed",
         action="store_true",
-        help=SUPPRESS,
+        help=SUPPRESS
     )
 
-    p.add_argument(
-        "--installed",
-        action="store_true",
-        help=SUPPRESS,
+    view_cmds = argparse.ArgumentParser(add_help=False)
+    view_cmds.add_argument(
+        "-t", "--tree",
+        action="store_true"
+    )
+
+    subsub_parser.add_parser('whoneeds',
+        help='shows packages that depends on this package',
+        parents=[package_cmds, view_cmds]
+    )
+
+    subsub_parser.add_parser('depends',
+        help='shows packages that depends on this package',
+        parents=[package_cmds, view_cmds]
+    )
+
+    subsub_parser.add_parser('search',
+        help='shows packages that depends on this package',
+        parents=[package_cmds]
     )
 
     from conda.cli import conda_argparse
     conda_argparse.add_parser_channels(p)
     conda_argparse.add_parser_networking(p)
     conda_argparse.add_parser_known(p)
+    conda_argparse.add_parser_json(p)
 
     p.set_defaults(func='.main_repoquery.execute')
     return p
 
-
 def _wrapped_main(*args, **kwargs):
     if len(args) == 1:
         args = args + ('-h',)
 
     import copy
     argv = list(args)
-    if "--mamba-download" in argv:
-        global use_mamba_download
-        use_mamba_download = True
-        argv.remove('--mamba-download')
-        args = argv
+
+    if "--mamba-experimental" in argv:
+        global use_mamba_experimental
+        use_mamba_experimental = True
+        argv.remove('--mamba-experimental')
+
+    args = argv
 
     p = generate_parser()
     configure_parser_repoquery(p._subparsers._group_actions[0])
@@ -729,6 +789,10 @@ def main(*args, **kwargs):
     if not args:
         args = sys.argv
 
+    if "--version" in args:
+        from mamba._version import __version__
+        print("mamba {}".format(__version__))
+
     args = tuple(ensure_text_type(s) for s in args)
 
     if len(args) > 2 and args[1] == 'env' and args[2] == 'create':
